/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/field widget UIM/X integration file
 *
 * $AstecId: XrtFld_xtype.c,v 1.8 2000/06/19 08:32:09 kosaka Exp $
 *
 *  SCCSID -- @(#)XrtFld_xtype.c	1.24    99/01/29 KL Group Inc.
 */

/******************************************************************************
				XrtFld_xtype.c
******************************************************************************/

#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#endif /* DESIGN_TIME */

#include "Xrt_restable.h"

#include <Xm/XrtField.h>

#include <cat_macros.h>

/* Begin new XRT/field types */

int		UxXT_XrtFldArrowType; /* pds */
int		UxXT_XrtFldCase;
int		UxXT_XrtFldDate;
int		UxXT_XrtFldDateIncrement;
int		UxXT_XrtFldDispScrollBar;
int		UxXT_XrtFldDouble;
int		UxXT_XrtFldError;
int		UxXT_XrtFldFieldType;
int		UxXT_XrtFldFormat;
int		UxXT_XrtFldMatchMask;
int 	UxXT_XrtFldMenuAlignment; /* pds */
int		UxXT_XrtFldMultiTypeList;
int		UxXT_XrtFldNegFormat;
int		UxXT_XrtFldPixel;
int		UxXT_XrtFldRange;
int 	UxXT_XrtFldSearchPolicy; /* pds */
int		UxXT_XrtFldSpinArrowLayout;
int		UxXT_XrtFldSpinType;
int		UxXT_XrtFldStringTable;

int     UxXT_XrtFldPopupType;
int     UxXT_XrtFldSpecialDays;
int     UxXT_XrtFldSpinPolicy;

/* Define three static arrays for each enumerated type */

/*  XrtFldArrowType */
static char * uXrtFldArrowType[] = {
    "Arrow_None",
	"Arrow_Wedge",
	"Arrow_With_Underbar",
	"Arrow_Custom"
};
static unsigned char xXrtFldArrowType[] = {
	XRTFLD_ARROW_NONE,
	XRTFLD_ARROW_WEDGE,
	XRTFLD_ARROW_WITH_UNDERBAR,
	XRTFLD_ARROW_CUSTOM
};
static char * dXrtFldArrowType[] = {
    "XRTFLD_ARROW_NONE",
	"XRTFLD_ARROW_WEDGE",
	"XRTFLD_ARROW_WITH_UNDERBAR",
	"XRTFLD_ARROW_CUSTOM"
};

/* XrtFldCase */
static char *uXrtFldCase[] = {
	"Case_As_Is",
	"Case_Lower",
	"Case_Upper"
};
static unsigned char xXrtFldCase[] = {
	XRTFLD_CASE_AS_IS,
	XRTFLD_CASE_LOWER,
	XRTFLD_CASE_UPPER
};
static char	*dXrtFldCase[] = {
	"XRTFLD_CASE_AS_IS",
	"XRTFLD_CASE_LOWER",
	"XRTFLD_CASE_UPPER"
};

/* XrtFldDispScrollBar */
static char *uXrtFldDispScrollBar[] = {
	"Dispsb_As_Needed",
	"Dispsb_Always",
	"Dispsb_Never"
};
static unsigned char xXrtFldDispScrollBar[] = {
	XRTFLD_DISPSB_AS_NEEDED,
	XRTFLD_DISPSB_ALWAYS,
	XRTFLD_DISPSB_NEVER
};
static char *dXrtFldDispScrollBar[] = {
	"XRTFLD_DISPSB_AS_NEEDED",
	"XRTFLD_DISPSB_ALWAYS",
	"XRTFLD_DISPSB_NEVER"
};

/* XrtFldError */
static char *uXrtFldError[] = {
	"Error_Exceed_Length",
	"Error_Exceed_Lines",
	"Error_Invalid_Char",
	"Error_Invalid_Datatype",
	"Error_Is_Null",
	"Error_Nomatch_Mask",
	"Error_Nomatch_Picklist",
	"Error_Out_Of_Range",
	"Error_Validatecb",
	"Error_None"
};
static unsigned char xXrtFldError[] = {
	XRTFLD_ERROR_EXCEED_LENGTH,
	XRTFLD_ERROR_EXCEED_LINES,
	XRTFLD_ERROR_INVALID_CHAR,
	XRTFLD_ERROR_INVALID_DATATYPE,
	XRTFLD_ERROR_IS_NULL,
	XRTFLD_ERROR_NOMATCH_MASK,
	XRTFLD_ERROR_NOMATCH_PICKLIST,
	XRTFLD_ERROR_OUT_OF_RANGE,
	XRTFLD_ERROR_VALIDATECB,
	XRTFLD_ERROR_NONE
};
static char	*dXrtFldError[] = {
	"XRTFLD_ERROR_EXCEED_LENGTH",
	"XRTFLD_ERROR_EXCEED_LINES",
	"XRTFLD_ERROR_INVALID_CHAR",
	"XRTFLD_ERROR_INVALID_DATATYPE",
	"XRTFLD_ERROR_IS_NULL",
	"XRTFLD_ERROR_NOMATCH_MASK",
	"XRTFLD_ERROR_NOMATCH_PICKLIST",
	"XRTFLD_ERROR_OUT_OF_RANGE",
	"XRTFLD_ERROR_VALIDATECB",
	"XRTFLD_ERROR_NONE"
};

/* XrtFldFieldType */
static char *uXrtFldFieldType[] = {
	"Fieldtype_Currency",
	"Fieldtype_Date",
	"Fieldtype_Float",
	"Fieldtype_Int",
	"Fieldtype_None",
	"Fieldtype_String"
};
static unsigned char xXrtFldFieldType[] = {
	XRTFLD_FIELDTYPE_CURRENCY,
	XRTFLD_FIELDTYPE_DATE,
	XRTFLD_FIELDTYPE_FLOAT,
	XRTFLD_FIELDTYPE_INT,
	XRTFLD_FIELDTYPE_NONE,
	XRTFLD_FIELDTYPE_STRING
};
static char	*dXrtFldFieldType[] = {
	"XRTFLD_FIELDTYPE_CURRENCY",
	"XRTFLD_FIELDTYPE_DATE",
	"XRTFLD_FIELDTYPE_FLOAT",
	"XRTFLD_FIELDTYPE_INT",
	"XRTFLD_FIELDTYPE_NONE",
	"XRTFLD_FIELDTYPE_STRING"
};

/* XrtFldFormat */
static char *uXrtFldFormat[] = {
	"Format_Binary",
	"Format_Currency",
	"Format_Currency_Intl",
	"Format_Default",
	"Format_Fixed",
	"Format_General",
	"Format_Hex",
	"Format_Octal",
	"Format_Percent",
	"Format_Scientific",
	"Format_Thousands"
};
static unsigned char xXrtFldFormat[] = {
	XRTFLD_FORMAT_BINARY,
	XRTFLD_FORMAT_CURRENCY,
	XRTFLD_FORMAT_CURRENCY_INTL,
	XRTFLD_FORMAT_DEFAULT,
	XRTFLD_FORMAT_FIXED,
	XRTFLD_FORMAT_GENERAL,
	XRTFLD_FORMAT_HEX,
	XRTFLD_FORMAT_OCTAL,
	XRTFLD_FORMAT_PERCENT,
	XRTFLD_FORMAT_SCIENTIFIC,
	XRTFLD_FORMAT_THOUSANDS
};
static char	*dXrtFldFormat[] = {
	"XRTFLD_FORMAT_BINARY",
	"XRTFLD_FORMAT_CURRENCY",
	"XRTFLD_FORMAT_CURRENCY_INTL",
	"XRTFLD_FORMAT_DEFAULT",
	"XRTFLD_FORMAT_FIXED",
	"XRTFLD_FORMAT_GENERAL",
	"XRTFLD_FORMAT_HEX",
	"XRTFLD_FORMAT_OCTAL",
	"XRTFLD_FORMAT_PERCENT",
	"XRTFLD_FORMAT_SCIENTIFIC",
	"XRTFLD_FORMAT_THOUSANDS"
};

/* XrtFldMatchMask */
static char *uXrtFldMatchMask[] = {
	"Match_All",
	"Match_Partial_L_To_R",
	"Match_Partial_R_To_L"
};
static unsigned char xXrtFldMatchMask[] = {
	XRTFLD_MATCH_ALL,
	XRTFLD_MATCH_PARTIAL_L_TO_R,
	XRTFLD_MATCH_PARTIAL_R_TO_L
};
static char	*dXrtFldMatchMask[] = {
	"XRTFLD_MATCH_ALL",
	"XRTFLD_MATCH_PARTIAL_L_TO_R",
	"XRTFLD_MATCH_PARTIAL_R_TO_L"
};

/*  XrtFldMenuAlignment */
static char * uXrtFldMenuAlignment[] = {
	"Menu_Align_Beginning",
	"Menu_Align_Both",
    "Menu_Align_End"
};
static unsigned char xXrtFldMenuAlignment[] = {
	XRTFLD_MENU_ALIGN_BEGINNING,
	XRTFLD_MENU_ALIGN_BOTH,
    XRTFLD_MENU_ALIGN_END
};
static char * dXrtFldMenuAlignment[] = {
	"XRTFLD_MENU_ALIGN_BEGINNING",
	"XRTFLD_MENU_ALIGN_BOTH",
    "XRTFLD_MENU_ALIGN_END"
};

/* XrtFldNegFormat */
static char *uXrtFldNegFormat[] = {
	"Neg_As_Is",
	"Neg_Locale",
	"Neg_Parentheses"
};
static unsigned char xXrtFldNegFormat[] = {
	XRTFLD_NEG_AS_IS,
	XRTFLD_NEG_LOCALE,
	XRTFLD_NEG_PARENTHESES
};
static char	*dXrtFldNegFormat[] = {
	"XRTFLD_NEG_AS_IS",
	"XRTFLD_NEG_LOCALE",
	"XRTFLD_NEG_PARENTHESES"
};

/*  XrtFldRange */
static char *uXrtFldRange[] = {
	"Range_Char",
	"Range_Default",
	"Range_Double",
	"Range_Float",
	"Range_Int",
	"Range_Long",
	"Range_Short",
	"Range_Unsigned_Char",
	"Range_Unsigned_Int",
	"Range_Unsigned_Long",
	"Range_Unsigned_Short"
};
static unsigned char xXrtFldRange[] = {
	XRTFLD_RANGE_CHAR,
	XRTFLD_RANGE_DEFAULT,
	XRTFLD_RANGE_DOUBLE,
	XRTFLD_RANGE_FLOAT,
	XRTFLD_RANGE_INT,
	XRTFLD_RANGE_LONG,
	XRTFLD_RANGE_SHORT,
	XRTFLD_RANGE_UNSIGNED_CHAR,
	XRTFLD_RANGE_UNSIGNED_INT,
	XRTFLD_RANGE_UNSIGNED_LONG,
	XRTFLD_RANGE_UNSIGNED_SHORT
};
static char	*dXrtFldRange[] = {
	"XRTFLD_RANGE_CHAR",
	"XRTFLD_RANGE_DEFAULT",
	"XRTFLD_RANGE_DOUBLE",
	"XRTFLD_RANGE_FLOAT",
	"XRTFLD_RANGE_INT",
	"XRTFLD_RANGE_LONG",
	"XRTFLD_RANGE_SHORT",
	"XRTFLD_RANGE_UNSIGNED_CHAR",
	"XRTFLD_RANGE_UNSIGNED_INT",
	"XRTFLD_RANGE_UNSIGNED_LONG",
	"XRTFLD_RANGE_UNSIGNED_SHORT"
};

/* XrtFldSearchPolicy */
static char * uXrtFldSearchPolicy[] = {
   "Search_Single",
   "Search_Multichar"
};
static unsigned char xXrtFldSearchPolicy[] = {
    XRTFLD_SEARCH_SINGLE,
    XRTFLD_SEARCH_MULTICHAR
};
static char * dXrtFldSearchPolicy[] = {
   "XRTFLD_SEARCH_SINGLE",
   "XRTFLD_SEARCH_MULTICHAR"
};

/* XrtFldSpinArrowLayout */
static char *uXrtFldSpinArrowLayout[] = {
	"Arrows_Beginning",
	"Arrows_End",
	"Arrows_Split"
};
static unsigned char xXrtFldSpinArrowLayout[] = {
	XRTSPIN_ARROWS_BEGINNING,
	XRTSPIN_ARROWS_END,
	XRTSPIN_ARROWS_SPLIT
};
static char	*dXrtFldSpinArrowLayout[] = {
	"XRTSPIN_ARROWS_BEGINNING",
	"XRTSPIN_ARROWS_END",
	"XRTSPIN_ARROWS_SPLIT"
};

/* XrtFldSpinType */
static char *uXrtFldSpinType[] = {
	"Type_Counter",
	"Type_Picklist"
};
static unsigned char xXrtFldSpinType[] = {
	XRTSPIN_TYPE_COUNTER,
	XRTSPIN_TYPE_PICKLIST
};
static char	*dXrtFldSpinType[] = {
	"XRTSPIN_TYPE_COUNTER",
	"XRTSPIN_TYPE_PICKLIST"
};

static char * uXrtFldPopupType[] = {
 	"Popup Picklist",
	"Popup Calendar"
};
static unsigned char xXrtFldPopupType[] = {
	XRTFLD_POPUP_PICKLIST,
	XRTFLD_POPUP_CALENDAR
};
static char * dXrtFldPopupType[] = {
	"XRTFLD_POPUP_PICKLIST",
	"XRTFLD_POPUP_CALENDAR"
};

static char * uXrtFldSpinPolicy[] = {
	"Spin_Field",
	"Spin_Subfield",
	"Spin_Wrap"
};
static unsigned char xXrtFldSpinPolicy[] = {
	XRTFLD_SPIN_FIELD,
	XRTFLD_SPIN_SUBFIELD,
	XRTFLD_SPIN_WRAP
};
static char * dXrtFldSpinPolicy[] = {
	"XRTFLD_SPIN_FIELD",
	"XRTFLD_SPIN_SUBFIELD",
	"XRTFLD_SPIN_WRAP"
};

#ifndef   RUNTIME

/*
 * ValuesOf functions for both enumerated types and other Xtypes
 */

int
UxValuesOfXrtFldArrowType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtFldArrowType;
	*n = XtNumber(uXrtFldArrowType);
	return(*n);
}

int
UxValuesOfXrtFldCase(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldCase;
	*n = XtNumber(uXrtFldCase);
	return(*n);
}


int
UxValuesOfXrtFldDate(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDDATE);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtFldDateIncrement(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDDATEINCREMENT);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtFldDispScrollBar(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldDispScrollBar;
	*n = XtNumber(uXrtFldDispScrollBar);
	return(*n);
}

int
UxValuesOfXrtFldDouble(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDDOUBLE);
	*s = values;
	*n = 1;

	return 0;
}

int
UxValuesOfXrtFldError(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldError;
	*n = XtNumber(uXrtFldError);
	return(*n);
}

int
UxValuesOfXrtFldFieldType(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldFieldType;
	*n = XtNumber(uXrtFldFieldType);
	return(*n);
}

int
UxValuesOfXrtFldFormat(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldFormat;
	*n = XtNumber(uXrtFldFormat);
	return(*n);
}

int
UxValuesOfXrtFldMatchMask(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldMatchMask;
	*n = XtNumber(uXrtFldMatchMask);
	return(*n);
}

int
UxValuesOfXrtFldMenuAlignment(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtFldMenuAlignment;
	*n = XtNumber(uXrtFldMenuAlignment);
	return(*n);
}

int
UxValuesOfXrtFldMultiTypeList(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDMULTITYPELIST);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtFldNegFormat(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldNegFormat;
	*n = XtNumber(uXrtFldNegFormat);
	return(*n);
}

int
UxValuesOfXrtFldPixel(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDPIXEL);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtFldRange(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldRange;
	*n = XtNumber(uXrtFldRange);
	return(*n);
}

int
UxValuesOfXrtFldSearchPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtFldSearchPolicy;
	*n = XtNumber(uXrtFldSearchPolicy);
	return(*n);
}

int
UxValuesOfXrtFldSpinArrowLayout(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldSpinArrowLayout;
	*n = XtNumber(uXrtFldSpinArrowLayout);
	return(*n);
}

int
UxValuesOfXrtFldSpinType(ulist, n)
char ***ulist;
int *n;
{
	*ulist = uXrtFldSpinType;
	*n = XtNumber(uXrtFldSpinType);
	return(*n);
}

int
UxValuesOfXrtFldStringTable(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_FLDSTRINGTABLE);
	*s = values;
	*n = 1;
	return 0;
}

/*
 * Validate functions for both enumerated types and other Xtypes.
 * Use a general function for enumerated types.
 */

static int
ValidateXrtFldEnumType(rs, s, string_list, n)
swidget rs;
char	*s;
char	**string_list;
int		n;
{
	int i;

	if (s == NULL) {
		return ERROR;
	}
	for (i = 0; i < n; i++) {
		if (UxStrEqual(s, string_list[i])) {
			return NO_ERROR;
		}
	}
	return ERROR;
}

int
UxValidateXrtFldArrowType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldArrowType,
								  XtNumber(uXrtFldArrowType)));
}

int
UxValidateXrtFldCase(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldCase,
								  XtNumber(uXrtFldCase)));
}

int
UxValidateXrtFldDispScrollBar(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldDispScrollBar,
								  XtNumber(uXrtFldDispScrollBar)));
}

int
UxValidateXrtFldError(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldError,
								  XtNumber(uXrtFldError)));
}

int
UxValidateXrtFldFieldType(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldFieldType,
								  XtNumber(uXrtFldFieldType)));
}

int
UxValidateXrtFldFormat(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldFormat,
								  XtNumber(uXrtFldFormat)));
}

int
UxValidateXrtFldMatchMask(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldMatchMask,
								  XtNumber(uXrtFldMatchMask)));
}

int
UxValidateXrtFldMenuAlignment(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldMenuAlignment,
								  XtNumber(uXrtFldMenuAlignment)));
}

#include <time.h>
int
UxValidateXrtFldDate(rs, s)
swidget	rs;
char	*s;
{
	struct tm	tm;

	if (UxValidateString(rs, s) == ERROR) {
		return ERROR;
	}
	if (strptime(s, "%Y-%m-%d %H:%M:%S", &tm) == NULL) {
		return ERROR;
	}
	else {
		return NO_ERROR;
	}
}

int
UxValidateXrtFldMultiTypeList(rs, s)
swidget	rs;
char	*s;
{
	char		*swgt_name;
	struct tm	tm;

	if (UxValidateString(rs, s) == ERROR) {
		return ERROR;
	}
	swgt_name = UxGetName(rs);
	if (strncmp(swgt_name, XRTDATEFLD_CLASS_NAME,
		    strlen(XRTDATEFLD_CLASS_NAME)) == 0) {
		if (strptime(s, "%Y-%m-%d %H:%M:%S", &tm) == NULL) {
			return ERROR;
		}
		else {
			return NO_ERROR;
		}
	}
}

int
UxValidateXrtFldNegFormat(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldNegFormat,
								  XtNumber(uXrtFldNegFormat)));
}

int
UxValidateXrtFldRange(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldRange,
								  XtNumber(uXrtFldRange)));
}

int
UxValidateXrtFldSearchPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldSearchPolicy,
								  XtNumber(uXrtFldSearchPolicy)));
}

int
UxValidateXrtFldSpinArrowLayout(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldSpinArrowLayout,
								  XtNumber(uXrtFldSpinArrowLayout)));
}

int
UxValidateXrtFldSpinType(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldSpinType,
								  XtNumber(uXrtFldSpinType)));
}


int
UxValuesOfXrtFldPopupType(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrtFldPopupType;
	*n = XtNumber(uXrtFldPopupType);
	return(*n);
}

int
UxValuesOfXrtFldSpecialDays(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VAOF_FLDSPECIALDAYS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtFldSpinPolicy(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrtFldSpinPolicy;
	*n = XtNumber(uXrtFldSpinPolicy);
	return(*n);
}

int
UxValidateXrtFldPopupType(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldPopupType,
								  XtNumber(uXrtFldPopupType)));
}

int
UxValidateXrtFldSpinPolicy(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtFldEnumType(rs, s, uXrtFldSpinPolicy,
								  XtNumber(uXrtFldSpinPolicy)));
}

#endif   /* ! RUNTIME */

/*
 *	Conversion functions TO and FROM UIMX
 */

static int
Long_Int(swgt, longval, intval, flag, xtype)
swidget	swgt;
long	*longval;
int		*intval;
int		flag;
int		xtype;
{
	int				status;
	XrmValue        from, to;
	char			buffer[128];

	status = NO_ERROR;

	if (flag == TO_UIMX) {

		*longval = (long) *intval;

		sprintf(buffer, "%ld", *longval);
	}
	else {

		*intval = (int) *longval;
		
		sprintf(buffer, "%d", *intval);
	}

done:
	
	return(status);
}

static int
String_Date(swgt, string, date, flag, xtype)
swidget			swgt;
char			**string;
struct tm		**date;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(struct tm *);
		from.addr = (caddr_t) date;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRxrtFldDate, &from,
							  XmRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a date string into a tm struct pointer
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(struct tm *);
		to.addr = (caddr_t) date;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRxrtFldDate, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_DateIncrement(swgt, string, date_incr, flag, xtype)
swidget			swgt;
char			**string;
struct tm		**date_incr;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(struct tm *);
		from.addr = (caddr_t) date_incr;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRxrtFldDateIncrement, &from,
							  XmRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a date string into a tm struct pointer
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(struct tm *);
		to.addr = (caddr_t) date_incr;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRxrtFldDateIncrement, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_MultiTypeList(swgt, string, pointer, flag, xtype)
swidget			swgt;
char			**string;
XtPointer		pointer;
int				flag;
int				xtype;
{
	int			status, size;
	XrmValue	from, to;
	char		*type, *swgt_name;

	status = ERROR;
	
	swgt_name = UxGetName(swgt);
	if (strncmp(swgt_name, XRTSTRINGFLD_CLASS_NAME,
				strlen(XRTSTRINGFLD_CLASS_NAME)) == 0)
	{
		type = XmRStringTable;
		size = sizeof(String *);
	}
	else if (strncmp(swgt_name, XRTDATEFLD_CLASS_NAME,
				strlen(XRTDATEFLD_CLASS_NAME)) == 0)
	{
		type = XmRxrtFldDateList;
		size = sizeof(struct tm **);
	}
	else if (strncmp(swgt_name, XRTINTFLD_CLASS_NAME,
				strlen(XRTINTFLD_CLASS_NAME)) == 0)
	{
		type = XmRxrtFldIntList;
		size = sizeof(int *);
	}
	else if (strncmp(swgt_name, XRTFLOATFLD_CLASS_NAME,
				strlen(XRTFLOATFLD_CLASS_NAME)) == 0)
	{
		type = XmRxrtFldFloatList;
		size = sizeof(double *);
	}
	else if (strncmp(swgt_name, XRTCURRENCYFLD_CLASS_NAME,
				strlen(XRTCURRENCYFLD_CLASS_NAME)) == 0)
	{
		type = XmRxrtFldFloatList;
		size = sizeof(double *);
	}
	else {
		printf("Bad widget name in String_MultiTypeList\n");
		goto done;
	}

	if (flag == TO_UIMX) {
		
		from.size = size;
		from.addr = (caddr_t) pointer;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  type, &from,
							  XmRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = size;
		to.addr = (caddr_t) pointer;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  type, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_Pixel(swgt, string, pixel, flag, xtype)
swidget			swgt;
char			**string;
Pixel			*pixel;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(Pixel);
		from.addr = (caddr_t) pixel;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRxrtFldPixel, &from,
							  XmRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a pixel value into a string
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Pixel);
		to.addr = (caddr_t) pixel;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRxrtFldPixel, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_StringTable(swgt, string, string_list, flag, xtype)
swidget			swgt;
char			**string;
char			***string_list;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(char **);
		from.addr = (caddr_t) string_list;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRStringTable, &from,
							  XmRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a string list into a list of longs
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(char **);
		to.addr = (caddr_t) string_list;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRStringTable, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_SpecialDays(swgt, string, sdays, flag, xtype)
swidget			swgt;
char			**string;
XrtCalendarSpecialDay	***sdays;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		from.size = sizeof(XrtCalendarSpecialDay **);
		from.addr = (caddr_t) sdays;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XmRXrtCalendarSpecialDays, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn a data styles string into a data styles structure
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtCalendarSpecialDay **);
		to.addr = (caddr_t) sdays;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtCalendarSpecialDays, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

void	
XrtFld_enum()
{
	UxXT_XrtFldArrowType =	UxAddEnumType(XmRXrtFldArrowType,
									sizeof(XrtFldArrowType),
									xXrtFldArrowType, 
									uXrtFldArrowType, 
									dXrtFldArrowType, 
									XtNumber(uXrtFldArrowType), 
									UxStringToCharEnum);
	UxXT_XrtFldCase = UxAddEnumType(XmRxrtFldCase,
									sizeof(XrtFldCase),
									xXrtFldCase,
									uXrtFldCase,
									dXrtFldCase,
									XtNumber(xXrtFldCase),
									UxStringToCharEnum);
	
	UxXT_XrtFldDispScrollBar = UxAddEnumType(XmRxrtFldDispScrollBar,
									 sizeof(XrtFldDispScrollBar),
									 xXrtFldDispScrollBar,
									 uXrtFldDispScrollBar,
									 dXrtFldDispScrollBar,
									 XtNumber(xXrtFldDispScrollBar),
									 UxStringToCharEnum);
	
	UxXT_XrtFldError = UxAddEnumType(XmRxrtFldError,
									 sizeof(XrtFldError),
									 xXrtFldError,
									 uXrtFldError,
									 dXrtFldError,
									 XtNumber(xXrtFldError),
									 UxStringToCharEnum);
	
	UxXT_XrtFldFieldType = UxAddEnumType(XmRxrtFldFieldType,
									 sizeof(XrtFldFieldType),
									 xXrtFldFieldType,
									 uXrtFldFieldType,
									 dXrtFldFieldType,
									 XtNumber(xXrtFldFieldType),
									 UxStringToCharEnum);
	
	UxXT_XrtFldFormat = UxAddEnumType(XmRxrtFldFormat,
									 sizeof(XrtFldFormat),
									 xXrtFldFormat,
									 uXrtFldFormat,
									 dXrtFldFormat,
									 XtNumber(xXrtFldFormat),
									 UxStringToCharEnum);
	
	UxXT_XrtFldMatchMask = UxAddEnumType(XmRxrtFldMatchMask,
									 sizeof(XrtFldMatchMask),
									 xXrtFldMatchMask,
									 uXrtFldMatchMask,
									 dXrtFldMatchMask,
									 XtNumber(xXrtFldMatchMask),
									 UxStringToCharEnum);
	
	UxXT_XrtFldMenuAlignment =	UxAddEnumType(XmRXrtFldMenuAlignment,
									sizeof(XrtFldMenuAlignment),
									xXrtFldMenuAlignment, 
									uXrtFldMenuAlignment, 
									dXrtFldMenuAlignment, 
									XtNumber(uXrtFldMenuAlignment), 
									UxStringToCharEnum);
	UxXT_XrtFldNegFormat = UxAddEnumType(XmRxrtFldNegativeFormat,
									 sizeof(XrtFldNegFormat),
									 xXrtFldNegFormat,
									 uXrtFldNegFormat,
									 dXrtFldNegFormat,
									 XtNumber(xXrtFldNegFormat),
									 UxStringToCharEnum);
	
	UxXT_XrtFldRange = UxAddEnumType(XmRxrtFldRange,
									 sizeof(XrtFldRange),
									 xXrtFldRange,
									 uXrtFldRange,
									 dXrtFldRange,
									 XtNumber(xXrtFldRange),
									 UxStringToCharEnum);
	
	UxXT_XrtFldSearchPolicy = UxAddEnumType(XmRXrtFldSearchPolicy, 
									 sizeof(XrtFldSearchPolicy),
									 xXrtFldSearchPolicy, 
									 uXrtFldSearchPolicy, 
									 dXrtFldSearchPolicy, 
									 XtNumber(uXrtFldSearchPolicy), 
									 UxStringToCharEnum); 
	UxXT_XrtFldSpinArrowLayout = UxAddEnumType(XmRxrtSpinArrowLayout,
									 sizeof(XrtSpinArrowLayout),
									 xXrtFldSpinArrowLayout,
									 uXrtFldSpinArrowLayout,
 								     dXrtFldSpinArrowLayout,
							  	     XtNumber(xXrtFldSpinArrowLayout),
								     UxStringToCharEnum);
	
	UxXT_XrtFldSpinType = UxAddEnumType(XmRxrtSpinType,
									sizeof(XrtSpinType),
									xXrtFldSpinType,
									uXrtFldSpinType,
									dXrtFldSpinType,
									XtNumber(xXrtFldSpinType),
									UxStringToCharEnum);
	
	UxXT_XrtFldPopupType = UxAddEnumType(XmRXrtFldPopupType, 
									sizeof(XrtFldPopupType),
									xXrtFldPopupType,
									uXrtFldPopupType,
									dXrtFldPopupType,
									XtNumber(xXrtFldPopupType),
									UxStringToCharEnum);
	UxXT_XrtFldSpinPolicy = UxAddEnumType(XmRXrtFldSpinPolicy, 
									sizeof(XrtFldSpinPolicy),
									xXrtFldSpinPolicy,
									uXrtFldSpinPolicy,
									dXrtFldSpinPolicy,
									XtNumber(xXrtFldSpinPolicy),
									UxStringToCharEnum);
}

/* Add new XRT/field xtypes */

void	
XrtFld_xtypes()
{
	/*
	 * Initialize widget class inorder to initialize converters
	 */

	XtInitializeWidgetClass(xmXrtFieldWidgetClass);
	XrtFldSetReverseConverters();

	/* Add new Xtypes */
	
	UxXT_XrtFldDate = UxAddXtype(XmRxrtFldDate, sizeof(struct tm *));
	UxXT_XrtFldDateIncrement = UxAddXtype(XmRxrtFldDateIncrement,
										  sizeof(struct tm *));
	UxXT_XrtFldDouble = UxAddXtype(XmRxrtFldFloat, sizeof(double));
	UxXT_XrtFldMultiTypeList = UxAddXtype("xrtFldMultiTypeList",
										  sizeof(XtPointer));
	UxXT_XrtFldPixel = UxAddXtype(XmRxrtFldPixel, sizeof(Pixel));
	UxXT_XrtFldStringTable = UxAddXtype(XmRStringTable, sizeof(char **));

	UxXT_XrtFldSpecialDays	= UxAddXtype(XmRXrtCalendarSpecialDays,
										 sizeof(XrtCalendarSpecialDay **));

	/* Add new converters for Xtypes */
	
	UxAddConv(UxUT_long,   UxXT_int,		     	 Long_Int);
	UxAddConv(UxUT_string, UxXT_XrtFldDate,   	     String_Date);
	UxAddConv(UxUT_string, UxXT_XrtFldDateIncrement, String_DateIncrement);
	UxAddConv(UxUT_string, UxXT_XrtFldMultiTypeList, String_MultiTypeList);
	UxAddConv(UxUT_string, UxXT_XrtFldPixel,   	     String_Pixel);
	UxAddConv(UxUT_string, UxXT_XrtFldStringTable,	 String_StringTable);
	
  	UxAddConv(UxUT_string, UxXT_XrtFldSpecialDays, 	String_SpecialDays);
}

/*
 * Special get function to replace UxGetProp in UxXrtFloatField.h
 * for resources of type "double". UxGetProp and UxDDGetProp
 * always return XtArgVal but XRT/field is somewhat asymetric
 * when it comes to putting and getting doubles. This funtion
 * gets the double (XRT style) then returns a pointer to it
 * which the UxGet... macro dereferences using XrtFldArgValToDouble -jw
 */

double
XrtFldArgValToDouble(v)
XtArgVal v;
{
    return(*(double *)v);
}


XtArgVal
UxGetXrtFldDoubleProp(sw, res)
swidget	sw;
char	*res;
{
  Widget          w;
  static double   value = (double) 0.0;

  if (UxIsSwidget(sw) && (w = (Widget) UxGetWidget(sw)) != NULL) {
    XtVaGetValues (w, res, &value, NULL);
  }

  return (XtArgVal)&value;
}
