/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/graph widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGph.cl.c	1.9    97/02/10 KL Group Inc.
 */

/*****************************************************************************
				XrtGph.cl.c
*****************************************************************************/

#include <Xm/Xm.h>				/* Motif header file */
#include <Xm/XrtGraph.h>		/* Widget-class public header file */

#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"				/* Visual Edge object system */
#include "valuesOf.h"			/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#include "XrtGph.cl.h"			/* private swidget-class header file */
#include "XrtGph_xtype.h"

/* Global variables for the XrtGraph class and its resource descriptors */

Class_t	UxC_XrtGraph = NULL_CLASS;

static void
XrtGph_define_resource(n, table, class)
int 				n;
XrtGphResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	XrtGphResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;
	
	switch(pResDesc->type->datatype.type_graph) {
	case XRTGPH_FLOAT:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_GETFUNCTION,	pResDesc->type->get_func,
							 RD_PUTFUNCTION,	pResDesc->type->put_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTGPH_DATA:
	case XRTGPH_DATA2:
	case XRTGPH_COLOR:
	case XRTGPH_FONT_LIST:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTGPH_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	default:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

/* Function that defines the XrtGraph class */

void
UxRegister_XrtGraph()
{
	int		n;
	
	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGraph = UxRegister_class("XrtGraph",
									UxC_primitive,
									sizeof(UxXrtGraph),
									sizeof(UxXrtGraphClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGraph, "UxXrtGraph.h");
	UxPutToolKitClass(UxC_XrtGraph, (char *) xtXrtGraphWidgetClass);
    UxPutIconBitmap(UxC_XrtGraph, "xrtGraph.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGraph );
	
	for (n = 0; n < numgph_res; n++) {
		if (xrtgph_res_table[n].used) {
			XrtGph_define_resource(n, xrtgph_res_table, UxC_XrtGraph);
		}
	}

}
