/*---------------------------------------------------------------------
 * $Source: /proj/uimx/XRTPDS/integrate-xrt/include/RCS/UxBase.h,v $
 * $Date: 1999/10/09 17:53:40 $             $Revision: 1.2 $
 * $AstecId: UxBase.h,v 1.4 2006/08/30 01:15:32 kosaka Exp $
 *---------------------------------------------------------------------
 *          Copyright (c) 1996, Visual Edge Software Ltd.
 *
 * ALL  RIGHTS  RESERVED.  Permission  to  use,  copy,  modify,  and
 * distribute  this  software  and its documentation for any purpose
 * and  without  fee  is  hereby  granted,  provided  that the above
 * copyright  notice  appear  in  all  copies  and  that  both  that
 * copyright  notice and this permission notice appear in supporting
 * documentation,  and that  the name of Visual Edge Software not be
 * used  in advertising  or publicity  pertaining to distribution of
 * the software without specific, written prior permission. The year
 * included in the notice is the year of the creation of the work.
 *---------------------------------------------------------------------
 * UxBase.h
 * -------------
 * Definition of the class UxBase.
 *
 * This class is the base class for both UxInterface and
 * UxSwidget. It defines the functionality common between
 * Components (interfaces) and Ux C++ Convenience Library
 * Motif wrapper classes.
 *--------------------------------------------------------------------*/

#ifndef _UXBASECLASS_INCLUDED
#define _UXBASECLASS_INCLUDED

#ifdef __cplusplus

class UxBase
{

   protected:
	swidget		UxThis;

   public:
	UXINLINE UxBase();
	UXINLINE ~UxBase();

	/* type conversion to swidget */
	UXINLINE operator swidget();
	UXINLINE operator swidget() const;

	/* type conversion to void* */
	/* equivalent to (void*) ((swidget) object) */
	UXINLINE operator void*();
	UXINLINE operator const void*() const;

	UXINLINE Widget GetWidget();
	UXINLINE int IsValidSwidget() const;
	UXINLINE swidget GetParent();

	UXINLINE int PopupInterface(grabtype grab = no_grab);
	UXINLINE int PopdownInterface();
	UXINLINE int DestroyInterface();
	UXINLINE swidget GetUxThis() const;

#ifndef XT_CODE

	/* set of functions only available in Ux mode */
	UXINLINE char* GetName() const;
        UXINLINE char* GetCreateManaged() const;
        UXINLINE void SetCreateManaged(const char* val);

	/* constraints supplied by the form and inherited by its kids */
	UXINLINE char *GetBottomAttachment() const;
	UXINLINE void SetBottomAttachment(const char *val);
	UXINLINE int GetBottomOffset() const;
	UXINLINE void SetBottomOffset(int val);
	UXINLINE int GetBottomPosition() const;
	UXINLINE void SetBottomPosition(int val);
	UXINLINE char *GetBottomWidget() const;
	UXINLINE void SetBottomWidget(const char *val);
	UXINLINE void SetBottomWidget(swidget val);
	UXINLINE char *GetLeftAttachment() const;
	UXINLINE void SetLeftAttachment(const char *val);
	UXINLINE int GetLeftOffset() const;
	UXINLINE void SetLeftOffset(int val);
	UXINLINE int GetLeftPosition() const;
	UXINLINE void SetLeftPosition(int val);
	UXINLINE char *GetLeftWidget() const;
	UXINLINE void SetLeftWidget(const char *val);
	UXINLINE void SetLeftWidget(swidget val);
	UXINLINE char *GetRightAttachment() const;
	UXINLINE void SetRightAttachment(const char *val);
	UXINLINE int GetRightOffset() const;
	UXINLINE void SetRightOffset(int val);
	UXINLINE int GetRightPosition() const;
	UXINLINE void SetRightPosition(int val);
	UXINLINE char *GetRightWidget() const;
	UXINLINE void SetRightWidget(const char *val);
	UXINLINE void SetRightWidget(swidget val);
	UXINLINE char *GetTopAttachment() const;
	UXINLINE void SetTopAttachment(const char *val);
	UXINLINE int GetTopOffset() const;
	UXINLINE void SetTopOffset(int val);
	UXINLINE int GetTopPosition() const;
	UXINLINE void SetTopPosition(int val);
	UXINLINE char *GetTopWidget() const; 
	UXINLINE void SetTopWidget(const char *val);
	UXINLINE void SetTopWidget(swidget val);

	/* supplied by both the form and the notebook */
	UXINLINE char *GetResizable() const;
	UXINLINE void SetResizable(const char *val);

	/* constraints supplied by the frame and inherited by it's children */
	UXINLINE char * GetChildVerticalAlignment() const;
	UXINLINE void   SetChildVerticalAlignment(const char * val);
	UXINLINE int    GetChildHorizontalSpacing() const;
	UXINLINE void   SetChildHorizontalSpacing(int val);
	UXINLINE char * GetChildHorizontalAlignment() const;
	UXINLINE void   SetChildHorizontalAlignment(const char * val);
	UXINLINE char * GetChildType() const;
	UXINLINE void   SetChildType(const char * val);
#ifdef linux
	UXINLINE char * GetFrameChildType() const;
	UXINLINE void   SetFrameChildType(const char * val);
#endif /* linux */

	/* constraints supplied by the pane and inherited by it's children */
	UXINLINE char * GetSkipAdjust() const;
	UXINLINE void   SetSkipAdjust(const char * val);
	UXINLINE int    GetPaneMinimum() const;
	UXINLINE void   SetPaneMinimum(int val);
	UXINLINE int    GetPaneMaximum() const;
	UXINLINE void   SetPaneMaximum(int val);
	UXINLINE char * GetAllowResize() const;
	UXINLINE void   SetAllowResize(const char * val);

	/* supplied by the pane, the rowcolumn and the container */
	UXINLINE short  GetPositionIndex() const;
	UXINLINE void   SetPositionIndex(short val);

#ifdef linux
	/* constraints supplied by the container and inherited by it's children */
	UXINLINE char *GetEntryParent() const; 
	UXINLINE void SetEntryParent(const char *val);
	UXINLINE void SetEntryParent(swidget val);
	UXINLINE char * GetOutlineState() const;
	UXINLINE void   SetOutlineState(const char * val);

	/* constraints supplied by the notebook and inherited by it's children */
	UXINLINE char * GetNotebookChildType() const;
	UXINLINE void   SetNotebookChildType(const char * val);
	UXINLINE int GetPageNumber() const;
	UXINLINE void SetPageNumber(int val);

	/* constraints supplied by the xmspinbox and inherited by it's children */
	UXINLINE char * GetArrowSensitivity() const;
	UXINLINE void   SetArrowSensitivity(const char * val);
	UXINLINE short  GetDecimalPoints() const;
	UXINLINE void   SetDecimalPoints(short val);
	UXINLINE int GetIncrementValue() const;
	UXINLINE void SetIncrementValue(int val);
	UXINLINE int GetMaximumValue() const;
	UXINLINE void SetMaximumValue(int val);
	UXINLINE int GetMinimumValue() const;
	UXINLINE void SetMinimumValue(int val);
	UXINLINE int GetNumValues() const;
	UXINLINE void SetNumValues(int val);
	UXINLINE int GetPosition() const;
	UXINLINE void SetPosition(int val);
	UXINLINE char * GetPositionType() const;
	UXINLINE void   SetPositionType(const char * val);
	UXINLINE char * GetSpinBoxChildType() const;
	UXINLINE void   SetSpinBoxChildType(const char * val);
        UXINLINE char * GetValues() const;
        UXINLINE void   SetValues(const char *);

#endif /* linux */

#ifdef sgi
	/* constraints supplied by the grid and inherited by it's children */
	UXINLINE char *GetResizeHorizontal() const;
	UXINLINE void SetResizeHorizontal(const char *val);
	UXINLINE char *GetResizeVertical() const;
	UXINLINE void SetResizeVertical(const char *val);
	UXINLINE int GetColumn() const;
	UXINLINE void SetColumn(int val);
	UXINLINE int GetRow() const;
	UXINLINE void SetRow(int val);
	UXINLINE char *GetGravity() const;
	UXINLINE void SetGravity(const char *val);

	/* constraints supplied by the springBox and inherited by it's children */
	UXINLINE int GetHorizontalSpring() const;
	UXINLINE void SetHorizontalSpring(int val);
	UXINLINE int GetVerticalSpring() const;
	UXINLINE void SetVerticalSpring(int val);
	UXINLINE int GetBottomSpring() const;
	UXINLINE void SetBottomSpring(int val);
	UXINLINE int GetTopSpring() const;
	UXINLINE void SetTopSpring(int val);
	UXINLINE int GetLeftSpring() const;
	UXINLINE void SetLeftSpring(int val);
	UXINLINE int GetRightSpring() const;
	UXINLINE void SetRightSpring(int val);

#endif /* sgi */

#ifdef XRT

	/* constraints supplied by XRT widgets and inherited by it's children */

		/* constraints supplied by the XrtAligner */
	UXINLINE char * GetXrtGearAlignerAlignment_Alnr (void) const;
	UXINLINE void SetXrtGearAlignerAlignment_Alnr (const char * val);
	UXINLINE char * GetXrtGearAlignBaseline_Alnr (void) const;
	UXINLINE void SetXrtGearAlignBaseline_Alnr (const char * val);
	UXINLINE int GetXrtGearPosition_Alnr (void) const;
	UXINLINE void SetXrtGearPosition_Alnr (int val);
	UXINLINE char * GetXrtGearResizeHeight_Alnr (void) const;
	UXINLINE void SetXrtGearResizeHeight_Alnr (const char * val);
	UXINLINE char * GetXrtGearResizeWidth_Alnr (void) const;
	UXINLINE void SetXrtGearResizeWidth_Alnr (const char * val);

		/* constraints supplied by the XrtToolbar */
	UXINLINE short GetXrtGearBottomSpace_Tbar (void) const;
	UXINLINE void SetXrtGearBottomSpace_Tbar (short val);
	UXINLINE short GetXrtGearLeftSpace_Tbar (void) const;
	UXINLINE void SetXrtGearLeftSpace_Tbar (short val);
	UXINLINE int GetXrtGearPosition_Tbar (void) const;
	UXINLINE void SetXrtGearPosition_Tbar (int val);
	UXINLINE short GetXrtGearRightSpace_Tbar (void) const;
	UXINLINE void SetXrtGearRightSpace_Tbar (short val);
	UXINLINE short GetXrtGearTopSpace_Tbar (void) const;
	UXINLINE void SetXrtGearTopSpace_Tbar (short val);

		/* constraints supplied by the XrtTable */
	UXINLINE int GetXrtTblBottomOffset (void) const;
	UXINLINE void SetXrtTblBottomOffset (int val);
	UXINLINE int GetXrtTblLeftOffset (void) const;
	UXINLINE void SetXrtTblLeftOffset (int val);
	UXINLINE char * GetXrtTblLocation (void) const;
	UXINLINE void SetXrtTblLocation (const char * val);
	UXINLINE int GetXrtTblRightOffset (void) const;
	UXINLINE void SetXrtTblRightOffset (int val);
	UXINLINE int GetXrtTblTopOffset (void) const;
	UXINLINE void SetXrtTblTopOffset (int val);
	UXINLINE char * GetXrtTblWidgetLocation (void) const;
	UXINLINE void SetXrtTblWidgetLocation (const char * val);
	UXINLINE char * GetXrtTblResizeToCell (void) const;
	UXINLINE void SetXrtTblResizeToCell(const char * val);


		/* constraints supplied by the XrtTabManager */
	UXINLINE char * GetXrtGearChildType_TabM (void) const;
	// read-only, so no SetXrtGearChildType_TabM
	UXINLINE int GetXrtGearPageNumber_TabM (void) const;
	UXINLINE void SetXrtGearPageNumber_TabM (int val);
	UXINLINE char * GetXrtGearPageWidget_TabM (void) const;
	UXINLINE void SetXrtGearPageWidget_TabM (const char * val);
//	UXINLINE int GetXrtGearWidgetPosition_TabM (void) const; 

// Constraint resources of XrtPanedWindow

	UXINLINE char * GetXrtGearAllowResize_PanedW (void) const;
	UXINLINE void SetXrtGearAllowResize_PanedW(const char * val);

	UXINLINE short GetXrtGearMaximum_PanedW (void) const;
	UXINLINE void SetXrtGearMaximum_PanedW(short val);

	UXINLINE short GetXrtGearMinimum_PanedW (void) const;
	UXINLINE void SetXrtGearMinimum_PanedW(short val);

	UXINLINE short GetXrtGearPosition_PanedW (void) const;
	UXINLINE void SetXrtGearPosition_PanedW(short val);

	UXINLINE char * GetXrtGearSkipAdjust_PanedW (void) const;
	UXINLINE void SetXrtGearSkipAdjust_PanedW(const char * val);

#endif /* XRT */

#ifdef DESIGN_TIME
	/* access to the convenience library used in DESIGN_TIME */
	int	GET_int(binptr object, const char *id) const;
	int	PUT_int(binptr object, const char *id, int val);
	char	GET_char(binptr object, const char *id) const;
	int	PUT_char(binptr object, const char *id, char val);
	short	GET_short(binptr object, const char *id) const;
	int	PUT_short(binptr object, const char *id, short val);
	long	GET_long(binptr object, const char *id) const;
	int	PUT_long(binptr object, const char *id, long val);
	double	GET_float(binptr object, const char *id) const;
	int	PUT_float(binptr object, const char *id, double val);
	stringTable GET_stringTable(binptr object, const char *id) const;
	int	GET_stringTable(binptr object, const char *id,
					stringTable val) const;
	voidFunction GET_voidFunction(binptr object, const char *id) const;
	int	PUT_voidFunction(binptr object, const char *id,
				voidFunction val);
	cardFunction GET_cardFunction(binptr object, const char *id) const;
	int PUT_cardFunction(binptr object, const char *id,
				cardFunction val);
	visualPointer GET_visualPointer(binptr object, const char *id) const;
	int	PUT_visualPointer(binptr object, const char *id,
				visualPointer val);
	XmTextSource GET_XmTextSource(binptr object, const char *id) const;
	int	PUT_XmTextSource(binptr object, const char *id,
				XmTextSource val);
	char	*GET_string(binptr object, const char *id) const;
	int	PUT_string(binptr object, const char *id, const char *val);
#else
	// access to the convenience library used in runtime
	inline char*	GetProp(const char *res) const;
	inline char	CharGetProp(const char *res) const;
	inline short	ShortGetProp(const char *res) const;
#ifdef Ux_LP64
	inline int	IntGetProp(const char *res) const;
#endif
	inline int	SetProp(const char *res, XtArgVal v);
	inline char*	DDGetProp(const char *val) const;
	inline int	DDSetProp(const char *val, XtArgVal v);
#endif /* DESIGN_TIME */
#endif /* XT_CODE */
};


#ifndef DESIGN_TIME

inline UxBase::UxBase()
{
	UxThis = NULL;
}

inline UxBase::~UxBase()
{
}

inline UxBase::operator swidget()
{
	return UxThis;
}

inline UxBase::operator swidget() const
{
	return UxThis;
}

inline UxBase::operator void*()
{
	return (void *) UxThis;
}

inline UxBase::operator const void*() const
{
	return (const void *) UxThis;
}

inline Widget UxBase::GetWidget() 
{
	return UxGetWidget(UxThis);
}

inline int UxBase::IsValidSwidget() const
{
	return UxIsValidSwidget(UxThis);
}

inline swidget UxBase::GetParent() 
{
	return UxGetParent(UxThis);
}

inline int UxBase::PopupInterface (grabtype grab)
{
	return UxPopupInterface(UxThis, grab);
}

inline int UxBase::PopdownInterface()
{
	return UxPopdownInterface(UxThis);
}

inline int UxBase::DestroyInterface()
{
	return UxDestroyInterface(UxThis);
}


inline swidget UxBase::GetUxThis() const
{
	return UxThis;
}

#ifndef XT_CODE
inline char* UxBase::GetName() const
{
	return UxGetName(UxThis);
}

inline char* UxBase::GetCreateManaged() const
{
	return UxGetCreateManaged(UxThis);
}

inline void UxBase::SetCreateManaged(const char* val)
{
	UxPutCreateManaged (UxThis, (char*)val);
}

inline char* UxBase::GetProp(const char *res) const
{
	return (char*) UxGetProp(UxThis, (char *) res);
}

inline char UxBase::CharGetProp(const char *res) const
{
	return (char) UxCharGetProp(UxThis, (char *) res);
}

inline short UxBase::ShortGetProp(const char *res) const
{
	return UxShortGetProp(UxThis, (char *) res);
}

inline int UxBase::SetProp(const char * res, XtArgVal val)
{
	return UxPutProp(UxThis, (char *) res, val);
}

inline char* UxBase::DDGetProp(const char *val) const
{
	return (char*) UxDDGetProp(UxThis, (char *) val);
}

inline int UxBase::DDSetProp(const char *res, XtArgVal val)
{
	return UxDDPutProp(UxThis, (char *) res, val);
}

#ifdef Ux_LP64
inline int UxBase::IntGetProp(const char *res) const
{
	return UxIntGetProp(UxThis, (char *) res);
}
#else
#define	IntGetProp	UX_POINTER_TO_INT_CAST GetProp
#endif

/* constraints supplied by the form and inherited by it kids */

#if defined(__alpha) && defined (__cplusplus)
#pragma message save
#pragma message disable narrowptr
#endif

inline char	*UxBase::GetBottomAttachment() const
{
	return DDGetProp(XmNbottomAttachment);
}

inline void	UxBase::SetBottomAttachment(const char *val)
{
	DDSetProp(XmNbottomAttachment, (XtArgVal) val);
}

inline int	UxBase::GetBottomOffset() const
{
	return (int)IntGetProp(XmNbottomOffset);
}

inline void	UxBase::SetBottomOffset(int val)
{
	SetProp(XmNbottomOffset, (XtArgVal) val);
}

inline int	UxBase::GetBottomPosition() const
{
	return (int)IntGetProp(XmNbottomPosition);
}

inline void	UxBase::SetBottomPosition(int val)
{
	SetProp(XmNbottomPosition, (XtArgVal) val);
}

inline char	*UxBase::GetBottomWidget() const
{
	return DDGetProp(XmNbottomWidget);
}

inline void	UxBase::SetBottomWidget(const char *val)
{
	DDSetProp(XmNbottomWidget, (XtArgVal) val);
}

inline void	UxBase::SetBottomWidget(swidget val)
{
	DDSetProp(XmNbottomWidget, (XtArgVal) val);
}

inline char	*UxBase::GetLeftAttachment() const
{
	return DDGetProp(XmNleftAttachment);
}

inline void	UxBase::SetLeftAttachment(const char *val)
{
	DDSetProp(XmNleftAttachment, (XtArgVal) val);
}

inline int	UxBase::GetLeftOffset() const
{
	return (int)IntGetProp(XmNleftOffset);
}

inline void	UxBase::SetLeftOffset(int val)
{
	SetProp(XmNleftOffset, (XtArgVal) val);
}

inline int	UxBase::GetLeftPosition() const
{
	return (int)IntGetProp(XmNleftPosition);
}

inline void	UxBase::SetLeftPosition(int val)
{
	SetProp(XmNleftPosition, (XtArgVal) val);
}

inline char	*UxBase::GetLeftWidget() const
{
	return DDGetProp(XmNleftWidget);
}

inline void	UxBase::SetLeftWidget(const char *val)
{
	DDSetProp(XmNleftWidget, (XtArgVal) val);
}

inline void	UxBase::SetLeftWidget(swidget val)
{
	DDSetProp(XmNleftWidget, (XtArgVal) val);
}

inline char	*UxBase::GetRightAttachment() const
{
	return DDGetProp(XmNrightAttachment);
}

inline void	UxBase::SetRightAttachment(const char *val)
{
	DDSetProp(XmNrightAttachment, (XtArgVal) val);
}

inline int	UxBase::GetRightOffset() const
{
	return (int)IntGetProp(XmNrightOffset);
}

inline void	UxBase::SetRightOffset(int val)
{
	SetProp(XmNrightOffset, (XtArgVal) val);
}

inline int	UxBase::GetRightPosition() const
{
	return (int)IntGetProp(XmNrightPosition);
}

inline void	UxBase::SetRightPosition(int val)
{
	SetProp(XmNrightPosition, (XtArgVal) val);
}

inline char	*UxBase::GetRightWidget() const
{
	return DDGetProp(XmNrightWidget);
}

inline void	UxBase::SetRightWidget(const char *val)
{
	DDSetProp(XmNrightWidget, (XtArgVal) val);
}

inline void	UxBase::SetRightWidget(swidget val)
{
	DDSetProp(XmNrightWidget, (XtArgVal) val);
}

inline char	*UxBase::GetTopAttachment() const
{
	return DDGetProp(XmNtopAttachment);
}

inline void	UxBase::SetTopAttachment(const char *val)
{
	DDSetProp(XmNtopAttachment, (XtArgVal) val);
}

inline int	UxBase::GetTopOffset() const
{
	return (int)IntGetProp(XmNtopOffset);
}

inline void	UxBase::SetTopOffset(int val)
{
	SetProp(XmNtopOffset, (XtArgVal) val);
}

inline int	UxBase::GetTopPosition() const
{
	return (int)IntGetProp(XmNtopPosition);
}

inline void	UxBase::SetTopPosition(int val)
{
	SetProp(XmNtopPosition, (XtArgVal) val);
}

inline char	*UxBase::GetTopWidget() const
{
	return DDGetProp(XmNtopWidget);
}

inline void	UxBase::SetTopWidget(const char *val)
{
	DDSetProp(XmNtopWidget,(XtArgVal) val);
}

inline void	UxBase::SetTopWidget(swidget val)
{
	DDSetProp(XmNtopWidget,(XtArgVal) val);
}

/* supplied by the form and the notebook */
inline char	*UxBase::GetResizable() const
{
	return DDGetProp(XmNresizable);
}

inline void	UxBase::SetResizable(const char *val)
{
	DDSetProp(XmNresizable, (XtArgVal) val);
}

/* constraints supplied by the frame and inherited by it's children */
inline char *	UxBase::GetChildVerticalAlignment() const
{
	return DDGetProp(XmNchildVerticalAlignment);
}

inline void	UxBase::SetChildVerticalAlignment(const char * val)
{
	DDSetProp(XmNchildVerticalAlignment, ((XtArgVal)(val)));
}

inline int	UxBase::GetChildHorizontalSpacing() const
{
	return UX_POINTER_TO_INT_CAST DDGetProp(XmNchildHorizontalSpacing);
}

inline void	UxBase::SetChildHorizontalSpacing(int val)
{
	DDSetProp(XmNchildHorizontalSpacing, ((XtArgVal)(val)));
}

inline char *	UxBase::GetChildHorizontalAlignment() const
{
	return DDGetProp(XmNchildHorizontalAlignment);
}

inline void	UxBase::SetChildHorizontalAlignment(const char * val)
{
	DDSetProp(XmNchildHorizontalAlignment, ((XtArgVal)(val)));
}

inline char *	UxBase::GetChildType() const
{
	return DDGetProp(XmNchildType);
}

inline void	UxBase::SetChildType(const char * val)
{
	DDSetProp(XmNchildType, ((XtArgVal)(val)));
}

#ifdef linux
inline char *	UxBase::GetFrameChildType() const
{
	return DDGetProp(XmNframeChildType);
}

inline void	UxBase::SetFrameChildType(const char * val)
{
	DDSetProp(XmNframeChildType, ((XtArgVal)(val)));
}
#endif /* linux */

/* constraints supplied by the pane and inherited by it's children */
inline char *	UxBase::GetSkipAdjust() const
{
	return DDGetProp(XmNskipAdjust);
}

inline void	UxBase::SetSkipAdjust(const char * val)
{
	DDSetProp(XmNskipAdjust, ((XtArgVal)(val)));
}

inline int	UxBase::GetPaneMinimum() const
{
	return UX_POINTER_TO_INT_CAST DDGetProp(XmNpaneMinimum);
}

inline void	UxBase::SetPaneMinimum(int val)
{
	DDSetProp(XmNpaneMinimum, ((XtArgVal)(val)));
}

inline int	UxBase::GetPaneMaximum() const
{
	return UX_POINTER_TO_INT_CAST DDGetProp(XmNpaneMaximum);
}

inline void	UxBase::SetPaneMaximum(int val)
{
	DDSetProp(XmNpaneMaximum, ((XtArgVal)(val)));
}

inline char *	UxBase::GetAllowResize() const
{
	return DDGetProp(XmNallowResize);
}

inline void	UxBase::SetAllowResize(const char * val)
{
	DDSetProp(XmNallowResize, ((XtArgVal)(val)));
}


/* supplied by the pane, the rowcolumn and the container */
inline short	UxBase::GetPositionIndex() const
{
	return (short)ShortGetProp(XmNpositionIndex);
}

inline void	UxBase::SetPositionIndex(short val)
{
	SetProp(XmNpositionIndex, ((XtArgVal)(val)));
}

#if defined(__alpha) && defined (__cplusplus)
#pragma message restore
#endif

#ifdef linux
/* constraints supplied by the container and inherited by it's children */

inline char	*UxBase::GetEntryParent() const
{
	return DDGetProp(XmNentryParent);
}

inline void	UxBase::SetEntryParent(const char *val)
{
	DDSetProp(XmNentryParent,(XtArgVal) val);
}

inline void	UxBase::SetEntryParent(swidget val)
{
	DDSetProp(XmNentryParent,(XtArgVal) val);
}

inline char *	UxBase::GetOutlineState() const
{
	return DDGetProp(XmNoutlineState);
}

inline void	UxBase::SetOutlineState(const char * val)
{
	DDSetProp(XmNoutlineState, ((XtArgVal)(val)));
}

/* constraints supplied by the notebook and inherited by it's children */

inline char *	UxBase::GetNotebookChildType() const
{
	return DDGetProp(XmNnotebookChildType);
}

inline void	UxBase::SetNotebookChildType(const char * val)
{
	DDSetProp(XmNnotebookChildType, ((XtArgVal)(val)));
}

inline int	UxBase::GetPageNumber() const
{
	return (int) IntGetProp(XmNpageNumber);
}

inline void	UxBase::SetPageNumber(int val)
{
	SetProp(XmNpageNumber, (XtArgVal) val);
}

/* constraints supplied by the xmspinbox and inherited by it's children */
inline char *	UxBase::GetArrowSensitivity() const
{
	return DDGetProp(XmNarrowSensitivity);
}

inline void	UxBase::SetArrowSensitivity(const char * val)
{
	DDSetProp(XmNarrowSensitivity, ((XtArgVal)(val)));
}

inline short	UxBase::GetDecimalPoints() const
{
	return (short)ShortGetProp(XmNdecimalPoints);
}

inline void	UxBase::SetDecimalPoints(short val)
{
	SetProp(XmNdecimalPoints, ((XtArgVal)(val)));
}

inline int	UxBase::GetIncrementValue() const
{
	return (int) IntGetProp(XmNincrementValue);
}

inline void	UxBase::SetIncrementValue(int val)
{
	SetProp(XmNincrementValue, (XtArgVal) val);
}

inline int	UxBase::GetMaximumValue() const
{
	return (int) IntGetProp(XmNmaximumValue);
}

inline void	UxBase::SetMaximumValue(int val)
{
	SetProp(XmNmaximumValue, (XtArgVal) val);
}

inline int	UxBase::GetMinimumValue() const
{
	return (int) IntGetProp(XmNminimumValue);
}

inline void	UxBase::SetMinimumValue(int val)
{
	SetProp(XmNminimumValue, (XtArgVal) val);
}

inline int	UxBase::GetNumValues() const
{
	return (int) IntGetProp(XmNnumValues);
}

inline void	UxBase::SetNumValues(int val)
{
	SetProp(XmNnumValues, (XtArgVal) val);
}

inline int	UxBase::GetPosition() const
{
	return (int) IntGetProp(XmNposition);
}

inline void	UxBase::SetPosition(int val)
{
	SetProp(XmNposition, (XtArgVal) val);
}

inline char *	UxBase::GetPositionType() const
{
	return DDGetProp(XmNpositionType);
}

inline void	UxBase::SetPositionType(const char * val)
{
	DDSetProp(XmNpositionType, ((XtArgVal)(val)));
}

inline char *	UxBase::GetSpinBoxChildType() const
{
	return DDGetProp(XmNspinBoxChildType);
}

inline void	UxBase::SetSpinBoxChildType(const char * val)
{
	DDSetProp(XmNspinBoxChildType, ((XtArgVal)(val)));
}
  
inline char *   UxBase::GetValues() const
{
	return (char *)DDGetProp(XmNvalues);
}

inline void     UxBase::SetValues(const char * val)
{
	DDSetProp(XmNvalues, ((XtArgVal)(val)));
}

#endif /* linux */

#ifdef sgi
/* constraints supplied by the grid and inherited by it kids */

inline char	*UxBase::GetResizeHorizontal() const
{
	return DDGetProp(XmNresizeHorizontal);
}

inline void	UxBase::SetResizeHorizontal(const char *val)
{
	DDSetProp(XmNresizeHorizontal, (XtArgVal) val);
}

inline char	*UxBase::GetResizeVertical() const
{
	return DDGetProp(XmNresizeVertical);
}

inline void	UxBase::SetResizeVertical(const char *val)
{
	DDSetProp(XmNresizeVertical, (XtArgVal) val);
}

inline int	UxBase::GetColumn() const
{
	return (int)IntGetProp(XmNcolumn);
}

inline void	UxBase::SetColumn(int val)
{
	SetProp(XmNcolumn, (XtArgVal) val);
}

inline int	UxBase::GetRow() const
{
	return (int)IntGetProp(XmNrow);
}

inline void	UxBase::SetRow(int val)
{
	SetProp(XmNrow, (XtArgVal) val);
}

inline char	*UxBase::GetGravity() const
{
	return DDGetProp(XmNgravity);
}

inline void	UxBase::SetGravity(const char *val)
{
	DDSetProp(XmNgravity, (XtArgVal) val);
}

/* constraints supplied by the springBox and inherited by it kids */

inline int	UxBase::GetHorizontalSpring() const
{
	return (int)IntGetProp(XmNhorizontalSpring);
}

inline void	UxBase::SetHorizontalSpring(int val)
{
	SetProp(XmNhorizontalSpring, (XtArgVal) val);
}

inline int	UxBase::GetVerticalSpring() const
{
	return (int)IntGetProp(XmNverticalSpring);
}

inline void	UxBase::SetVerticalSpring(int val)
{
	SetProp(XmNverticalSpring, (XtArgVal) val);
}

inline int	UxBase::GetBottomSpring() const
{
	return (int)IntGetProp(XmNbottomSpring);
}

inline void	UxBase::SetBottomSpring(int val)
{
	SetProp(XmNbottomSpring, (XtArgVal) val);
}

inline int	UxBase::GetTopSpring() const
{
	return (int)IntGetProp(XmNtopSpring);
}

inline void	UxBase::SetTopSpring(int val)
{
	SetProp(XmNtopSpring, (XtArgVal) val);
}

inline int	UxBase::GetLeftSpring() const
{
	return (int)IntGetProp(XmNleftSpring);
}

inline void	UxBase::SetLeftSpring(int val)
{
	SetProp(XmNleftSpring, (XtArgVal) val);
}

inline int	UxBase::GetRightSpring() const
{
	return (int)IntGetProp(XmNrightSpring);
}

inline void	UxBase::SetRightSpring(int val)
{
	SetProp(XmNrightSpring, (XtArgVal) val);
}

#endif  /* sgi */

#ifdef XRT
	/* include UxXrtBase.h which contains definitions
	 * for constraint methods supplied by XRT widgets
	 */
#include "UxXrtBase.h"
#endif /* XRT */

#endif	/* XT_CODE */
#endif	/* ! DESIGN_TIME */
#endif	/* __cplusplus */
#endif	/* _UXBASECLASS_INCLUDED */
