/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/table widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtTbl.cl.c	1.11    97/02/10 KL Group Inc.
 */

/*****************************************************************************
				XrtTbl.cl.c
*****************************************************************************/

#include <Xm/Xm.h>				/* Motif header file */
#include <Xm/XrtTable.h>		/* Widget-class public header file */

#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#include "XrtTbl.cl.h"			/* private swidget-class header file */
#include "XrtTbl_xtype.h"

/* Global variables for the XRT/table widgets and its resource descriptors */

Class_t	UxC_XrtTable = NULL_CLASS;
Class_t	UxC_XrtLabel = NULL_CLASS;


static void
XrtTbl_define_resource(n, table, class)
int 				n;
XrtTblResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	XrtTblResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;
	
	switch(pResDesc->type->datatype.type_table) {
	case XRTTBL_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;

	case XRTTBL_WIDGET_CLASS_LIST:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_FORMAT,		"(char *)0x%lx",
							 RD_END);
		break;
	default:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

/* Function that defines the XRT/table class */

void
UxRegister_XrtTable()
{
	int		n;
	
	if (UxUndefined_class(UxC_manager)) {
		UxRegister_manager();
	}

	UxC_XrtTable = UxRegister_class("XrtTable",
									UxC_manager,
									sizeof(UxXrtTable),
									sizeof(UxXrtTableClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtTable, "UxXrtTable.h" );
	UxPutToolKitClass(UxC_XrtTable, (char *) xtXrtTableWidgetClass );
	UxPutIconBitmap(UxC_XrtTable, "xrtTable.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtTable );

	for (n = 0; n < numtbl_res; n++) {
		if (xrttbl_res_table[n].used) {
			XrtTbl_define_resource(n, xrttbl_res_table, UxC_XrtTable);
		}
	}

}

/* Function that defines the XrtLabel class */

void
UxRegister_XrtLabel()
{
	int		n;
	
	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtLabel = UxRegister_class("XrtLabel",
									UxC_primitive,
									sizeof(UxXrtLabel),
									sizeof(UxXrtLabelClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtLabel, "UxXrtLabel.h" );
	UxPutToolKitClass(UxC_XrtLabel, (char *) xtXrtLabelWidgetClass );
	UxPutIconBitmap(UxC_XrtLabel, "xrtLabel.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtLabel );

	for (n = 0; n < numlbl_res; n++) {
		if (xrtlbl_res_table[n].used) {
			XrtTbl_define_resource(n, xrtlbl_res_table, UxC_XrtLabel);
		}
	}

}
